@@ -3,7 +3,7 @@
 use std::fmt::Display;
 
 use crate::collections::handle_table::Handle;
-use crate::prelude::CardIndex;
+use crate::prelude::Trace;
 use crate::traits::VmFunction;
 use thiserror::Error;
 
@@ -19,7 +19,7 @@ pub enum ExecutionErrorPayload {
     ExitCode(i32),
     #[error("Got an invalid instruction code {0}")]
     InvalidInstruction(u8),
-    #[error("Got an invalid argument to function call; {}",
+    #[error("Got an invalid argument: {}",
         .context.as_ref().map(|x|x.as_str()).unwrap_or_else(|| ""))]
     InvalidArgument { context: Option<String> },
     #[error("Variable {0} was not found!")]
@@ -52,11 +52,11 @@ pub enum ExecutionErrorPayload {
 #[derive(Debug, Clone, Error)]
 pub struct ExecutionError {
     pub payload: ExecutionErrorPayload,
-    pub trace: Vec<CardIndex>,
+    pub trace: Vec<Trace>,
 }
 
 impl ExecutionError {
-    pub fn new(payload: ExecutionErrorPayload, trace: Vec<CardIndex>) -> Self {
+    pub fn new(payload: ExecutionErrorPayload, trace: Vec<Trace>) -> Self {
         Self { payload, trace }
     }
 }
