@@ -7,7 +7,7 @@ pub mod runtime;
 #[cfg(test)]
 mod tests;
 
-use self::runtime::CallFrame;
+use self::runtime::{cao_lang_object::ObjectGcGuard, CallFrame};
 use crate::{
     collections::handle_table::{Handle, HandleTable},
     instruction::Instruction,
@@ -16,7 +16,7 @@ use crate::{
     VariableId,
 };
 use runtime::RuntimeData;
-use std::{mem::transmute, str::FromStr};
+use std::{mem::transmute, ops::DerefMut, pin::Pin, str::FromStr};
 use tracing::debug;
 
 /// Cao-Lang bytecode interpreter.
@@ -29,7 +29,7 @@ where
     /// Number of instructions `run` will execute before returning Timeout
     pub max_instr: u64,
 
-    pub runtime_data: RuntimeData,
+    pub runtime_data: Pin<Box<RuntimeData>>,
 
     callables: HandleTable<Procedure<Aux>>,
     _m: std::marker::PhantomData<&'a ()>,
@@ -52,18 +52,17 @@ impl<'a, Aux> Vm<'a, Aux> {
             OwnedValue::Nil => Value::Nil,
             OwnedValue::String(s) => {
                 let res = self.init_string(s.as_str())?;
-                Value::String(res)
+                Value::Object(res.0)
             }
-            OwnedValue::Object(o) => {
+            OwnedValue::Table(o) => {
                 let mut res = self.init_table()?;
+                let table = res.deref_mut().as_table_mut().unwrap();
                 for OwnedEntry { key, value } in o.iter() {
                     let key = self.insert_value(key)?;
                     let value = self.insert_value(value)?;
-                    unsafe {
-                        res.as_mut().insert(key, value)?;
-                    }
+                    table.insert(key, value)?;
                 }
-                Value::Object(res.as_ptr())
+                Value::Object(res.0)
             }
             OwnedValue::Integer(x) => Value::Integer(*x),
             OwnedValue::Real(x) => Value::Real(*x),
@@ -143,7 +142,11 @@ impl<'a, Aux> Vm<'a, Aux> {
 
     pub fn get_table(&self, value: Value) -> Result<&CaoLangTable, ExecutionErrorPayload> {
         let res = match value {
-            Value::Object(o) => unsafe { &*o },
+            Value::Object(o) => unsafe {
+                o.as_ref()
+                    .as_table()
+                    .ok_or_else(|| ExecutionErrorPayload::invalid_argument("Expected Table"))?
+            },
             _ => {
                 debug!("Got {:?} instead of object", value);
                 return Err(ExecutionErrorPayload::invalid_argument("Expected Table"));
@@ -154,7 +157,11 @@ impl<'a, Aux> Vm<'a, Aux> {
 
     pub fn get_table_mut(&self, value: Value) -> Result<&mut CaoLangTable, ExecutionErrorPayload> {
         let res = match value {
-            Value::Object(o) => unsafe { &mut *o },
+            Value::Object(mut o) => unsafe {
+                o.as_mut()
+                    .as_table_mut()
+                    .ok_or_else(|| ExecutionErrorPayload::invalid_argument("Expected Table"))?
+            },
             _ => {
                 debug!("Got {:?} instead of object", value);
                 return Err(ExecutionErrorPayload::invalid_argument("Expected Table"));
@@ -165,26 +172,13 @@ impl<'a, Aux> Vm<'a, Aux> {
 
     /// Initializes a new FieldTable in this VM instance
     #[inline]
-    pub fn init_table(&mut self) -> Result<std::ptr::NonNull<CaoLangTable>, ExecutionErrorPayload> {
+    pub fn init_table(&mut self) -> Result<ObjectGcGuard, ExecutionErrorPayload> {
         self.runtime_data.init_table()
     }
 
     /// Initializes a new string owned by this VM instance
-    pub fn init_string(&mut self, payload: &str) -> Result<StrPointer, ExecutionErrorPayload> {
-        unsafe {
-            let layout = std::alloc::Layout::from_size_align(4 + payload.len(), 4).unwrap();
-            let mut ptr = self
-                .runtime_data
-                .memory
-                .alloc(layout)
-                .map_err(|_| ExecutionErrorPayload::OutOfMemory)?;
-
-            let result: *mut u8 = ptr.as_mut();
-            std::ptr::write(result as *mut u32, payload.len() as u32);
-            std::ptr::copy(payload.as_ptr(), result.add(4), payload.len());
-
-            Ok(StrPointer(ptr.as_ptr()))
-        }
+    pub fn init_string(&mut self, payload: &str) -> Result<ObjectGcGuard, ExecutionErrorPayload> {
+        self.runtime_data.init_string(payload)
     }
 
     /// This mostly assumes that program is valid, produced by the compiler.
@@ -232,19 +226,15 @@ impl<'a, Aux> Vm<'a, Aux> {
             let instr: Instruction = unsafe { transmute(instr) };
             let src_ptr = instr_ptr;
             instr_ptr += 1;
-            debug!(
-                "Executing: {:?} instr_ptr: {} Stack: {}",
-                instr, instr_ptr, self.runtime_data.value_stack
-            );
+            debug!("Executing: {instr:?} instr_ptr: {instr_ptr}");
             match instr {
                 Instruction::InitTable => {
                     let res = self.init_table().map_err(|err| {
                         payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
                     })?;
-                    self.stack_push(Value::Object(res.as_ptr()))
-                        .map_err(|err| {
-                            payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
-                        })?;
+                    self.stack_push(Value::Object(res.0)).map_err(|err| {
+                        payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                    })?;
                 }
                 Instruction::GetProperty => {
                     let key = self.stack_pop();
@@ -318,13 +308,13 @@ impl<'a, Aux> Vm<'a, Aux> {
                     payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
                 })?,
                 Instruction::ClearStack => {
-                    self.runtime_data.value_stack.clear_until(
-                        self.runtime_data
-                            .call_stack
-                            .last()
-                            .expect("No callframe available")
-                            .stack_offset as usize,
-                    );
+                    let offset = self
+                        .runtime_data
+                        .call_stack
+                        .last()
+                        .expect("No callframe available")
+                        .stack_offset as usize;
+                    self.runtime_data.value_stack.clear_until(offset);
                 }
                 Instruction::SetLocalVar => {
                     instr_execution::set_local(self, &program.bytecode, &mut instr_ptr).map_err(
@@ -473,7 +463,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                         |err| payload_to_error(err, instr_ptr, &self.runtime_data.call_stack),
                     )?
                 }
-                Instruction::Call => {
+                Instruction::CallNative => {
                     instr_execution::execute_call(self, &mut instr_ptr, &program.bytecode).map_err(
                         |err| payload_to_error(err, instr_ptr, &self.runtime_data.call_stack),
                     )?
@@ -538,6 +528,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                     })?;
                 }
             }
+            debug!("Stack: {}", self.runtime_data.value_stack);
         }
 
         Err(payload_to_error(
