@@ -1,7 +1,7 @@
 use std::convert::TryFrom;
 
 use bytemuck::Pod;
-use tracing::debug;
+use tracing::{debug, trace};
 
 use crate::{
     bytecode::{decode_str, read_from_bytes},
@@ -84,18 +84,7 @@ pub fn instr_len<T>(vm: &mut Vm<T>) -> ExecutionResult {
     let len = match val {
         Value::Nil => 0,
         Value::Function { .. } | Value::Integer(_) | Value::Real(_) => 1,
-        Value::String(s) => {
-            let st = unsafe {
-                s.get_str().ok_or_else(|| {
-                    ExecutionErrorPayload::invalid_argument("String not found".to_string())
-                })?
-            };
-            st.len() as i64
-        }
-        Value::Object(t) => {
-            let t = unsafe { &*t };
-            t.len() as i64
-        }
+        Value::Object(t) => unsafe { t.as_ref().len() as i64 },
     };
     vm.stack_push(len)?;
     Ok(())
@@ -111,7 +100,7 @@ pub fn instr_string_literal<T>(
         .ok_or(ExecutionErrorPayload::InvalidArgument { context: None })?;
 
     let ptr = vm.init_string(payload)?;
-    vm.stack_push(Value::String(ptr))?;
+    vm.stack_push(Value::Object(ptr.0))?;
 
     Ok(())
 }
@@ -236,10 +225,10 @@ pub fn instr_return<T>(vm: &mut Vm<T>, instr_ptr: &mut usize) -> ExecutionResult
     // pop the current stack frame
     let value = match vm.runtime_data.call_stack.pop() {
         // return value
-        Some(rt) => vm
+        Some(call_frame) => vm
             .runtime_data
             .value_stack
-            .clear_until(rt.stack_offset as usize),
+            .clear_until(call_frame.stack_offset as usize),
         None => {
             return Err(ExecutionErrorPayload::BadReturn {
                 reason: "Call stack is empty".to_string(),
@@ -260,6 +249,7 @@ pub fn instr_return<T>(vm: &mut Vm<T>, instr_ptr: &mut usize) -> ExecutionResult
         }
     }
     // push the return value
+    trace!("Return {value:?}");
     vm.stack_push(value)?;
     Ok(())
 }
